//
//  XMFullscreenImageViewController.m
//  TFM
//
//  Created by John Wu on 6/5/12.
//  Copyright (c) 2012 TFM. All rights reserved.
//

#import "XMFullscreenImageViewController.h"
#import "XMZoomingScrollView.h"
#import "XMPhoto.h"

@interface XMFullscreenImageViewController ()

@property(nonatomic, strong) XMZoomingScrollView *scrollView;
@property(nonatomic, strong) XMTapDetectingImageView *imageView;

@end

@implementation XMFullscreenImageViewController

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (id)init {
    self = [super init];
    if (self) {
        self.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    }
    return self;
}

- (id)initWithDefaultImage:(UIImage *)defaultImage url:(NSString *)url {
    self = [self init];
    if (self) {
        self.defaultImage = defaultImage;
        self.photo = [[XMPhoto alloc] initWithURL:[NSURL URLWithString:url]];
    }

    return self;
}

#pragma mark - View Life Cycle

- (void)loadView {
    self.view = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.view.backgroundColor = [UIColor blackColor];

    self.scrollView = [[XMZoomingScrollView alloc] initWithFrame:self.view.bounds photoBrowser:self];
    self.scrollView.autoresizingMask = self.view.autoresizingMask;
    self.scrollView.showPhotoHeightFull = self.showPhotoHeightFull;
    [self.view addSubview:self.scrollView];

    self.imageView = [self.scrollView photoImageView];
    self.imageView.autoresizingMask = self.view.autoresizingMask;
    self.imageView.userInteractionEnabled = YES;

    self.imageView.image = _defaultImage;
    self.scrollView.photo = self.photo;

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(toggleControls)
                                                 name:NFCXPhotoImageCancel
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleCXPhotoImageDidStartLoad:)
                                                 name:NFCXPhotoImageDidStartLoad
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleCXPhotoImageDidFinishLoad:)
                                                 name:NFCXPhotoImageDidFinishLoad
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleCXPhotoImageDidFailLoadWithError:)
                                                 name:NFCXPhotoImageDidFailLoadWithError
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleCXPhotoImageDidStartReload:)
                                                 name:NFCXPhotoImageDidStartReload
                                               object:nil];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    UIApplication *app = [UIApplication sharedApplication];
    [app setStatusBarHidden:YES withAnimation:UIStatusBarAnimationFade];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];

    UIApplication *app = [UIApplication sharedApplication];
    [app setStatusBarHidden:NO withAnimation:UIStatusBarAnimationFade];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Notify Handle
- (void)handleCXPhotoImageDidStartLoad:(NSNotification *)notification {
    id <XMPhotoProtocol> photo = [notification object];

    if ([photo underlyingImage]) {
        [self.scrollView displayImage];
    }
    else {
        [self.scrollView displayImageStartLoading];
    }
}

- (void)handleCXPhotoImageDidFinishLoad:(NSNotification *)notification {
    id <XMPhotoProtocol> photo = [notification object];
    if (self.scrollView) {
        if ([photo underlyingImage]) {
            // Successful load
            [self.scrollView displayImage];
        } else {
            // Failed to load
            [self.scrollView displayImageFailure];
        }
    }
}

- (void)handleCXPhotoImageDidFailLoadWithError:(NSNotification *)notification {
    [self.scrollView displayImageFailure];
}

- (void)handleCXPhotoImageDidStartReload:(NSNotification *)notification {
    id <XMPhotoProtocol> photo = [notification object];
    [photo loadUnderlyingImageAndNotify];
}

- (void)toggleControls {
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Rotation;
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    return YES;
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
	// Remember page index before rotation
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    [self.photo loadUnderlyingImageAndNotify];
}

@end
