//
//  XMPhotoLoadingView.m
//  CXPhotoBrowserDemo
//
//  Created by ChrisXu on 13/4/22.
//  Copyright (c) 2013年 ChrisXu. All rights reserved.
//

#import "XMPhotoLoadingView.h"
#import <QuartzCore/QuartzCore.h>
#import "XMPhoto.h"

@interface XMPhotoLoadingView ()
{    
    UIButton *reloadButton;
    UILabel *failureLabel;
}
@property (nonatomic, assign) XMPhoto *photo;
@property (nonatomic, strong) UIActivityIndicatorView *indicator;

@end

@implementation XMPhotoLoadingView
@synthesize photo = _photo;
@synthesize supportReload = _supportReload;

- (id)initWithPhoto:(XMPhoto *)photo
{
    self = [super init];
    if (self)
    {
        _photo = photo;
        reloadButton = [UIButton buttonWithType:UIButtonTypeCustom];
        failureLabel = [[UILabel alloc] init];
    }
    return self;
}
#pragma mark - PV

- (void)displayLoading
{
    [reloadButton removeFromSuperview];
    [failureLabel removeFromSuperview];
    [self addLoading];
}

- (void)addLoading {
    if (!self.indicator)
    {
        self.indicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
        [self.indicator setCenter:self.center];
        [self.indicator setHidesWhenStopped:YES];
        [self addSubview:self.indicator];
    }
    [self.indicator startAnimating];
}

- (void)displayFailure
{
    [self.indicator stopAnimating];
    
    [reloadButton.titleLabel setFont:[UIFont boldSystemFontOfSize:12.]];
    [reloadButton setTitle:@"Reload" forState:UIControlStateNormal];
    [reloadButton setFrame:CGRectMake(20, 10, 100, 30)];
    [reloadButton setCenter:self.center];
    [reloadButton addTarget:self.photo action:@selector(reloadImage) forControlEvents:UIControlEventTouchUpInside];
    [reloadButton.layer setMasksToBounds:YES];
    [reloadButton.layer setCornerRadius:4.0];
    [reloadButton.layer setBorderWidth:1.0];
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGColorRef colorref = CGColorCreate(colorSpace,(CGFloat[]){ 1, 1, 1, 1 });
    [reloadButton.layer setBorderColor:colorref];
    reloadButton.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [self addSubview:reloadButton];
    
    [reloadButton setHidden:!_supportReload];
    
    [failureLabel setFrame:CGRectMake(CGRectGetMidX(reloadButton.frame) - self.bounds.size.width/2, CGRectGetMinY(reloadButton.frame) - 60, self.bounds.size.width, 44)];
    [failureLabel setNumberOfLines:0.0];
    [failureLabel setTextAlignment:NSTextAlignmentCenter];
    [failureLabel setText:@"Sorry ! Unable to load image."];
    [failureLabel setFont:[UIFont boldSystemFontOfSize:20.]];
    [failureLabel setTextColor:[UIColor whiteColor]];
    [failureLabel setBackgroundColor:[UIColor clearColor]];
    failureLabel.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [self addSubview:failureLabel];
}

#pragma mark - Touched
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	NSUInteger tapCount = touch.tapCount;
	switch (tapCount) {
		case 1:
			[self handleSingleTap:touch];
			break;
		case 2:
			[self handleDoubleTap:touch];
			break;
		case 3:
			[self handleTripleTap:touch];
			break;
		default:
			break;
	}
    
	[[self nextResponder] touchesEnded:touches withEvent:event];
}

- (void)handleSingleTap:(UITouch *)touch
{
    [self.photo cancelImage];
}

- (void)handleDoubleTap:(UITouch *)touch
{
    [self.photo cancelImage];
}

- (void)handleTripleTap:(UITouch *)touch
{
    [self.photo cancelImage]; 
}

@end
