//
//  XMZoomingScrollView.m
//  CXPhotoBrowserDemo
//
//  Created by ChrisXu on 13/4/19.
//  Copyright (c) 2013年 ChrisXu. All rights reserved.
//

#import "XMZoomingScrollView.h"
#import "XMFullscreenImageViewController.h"

@interface XMZoomingScrollView ()
- (UIImage *)imageForPhoto:(id<XMPhotoProtocol>)photo;

@end

@interface XMZoomingScrollView ()
{
    CGFloat zoomScaleFromInit;
    BOOL shouldSupportedPanGesture;
}
@property (nonatomic, assign) XMFullscreenImageViewController *photoBrowser;

- (void)layoutPhotoLoadingView;

@end

@implementation XMZoomingScrollView

@synthesize photoBrowser = _photoBrowser, photo = _photo;
@synthesize photoImageView = _photoImageView;

- (id)initWithFrame:(CGRect)frame photoBrowser:(XMFullscreenImageViewController *)photoBrowser {
    self = [self initWithFrame:frame];
    if (self) {
        self.isPhotoSupportedReload = YES;
        self.isPhotoSupportedPanGesture = YES;
        self.showPhotoHeightFull = YES;
        self.photoBrowser = photoBrowser;
        
		// Tap view for background
		_tapView = [[XMTapDetectingView alloc] initWithFrame:self.bounds];
		_tapView.tapDelegate = self;
		_tapView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		_tapView.backgroundColor = [UIColor blackColor];
		[self addSubview:_tapView];
		
		// Image view
		_photoImageView = [[XMTapDetectingImageView alloc] initWithFrame:self.bounds];
		_photoImageView.tapDelegate = self;
		_photoImageView.contentMode = UIViewContentModeCenter;
		_photoImageView.backgroundColor = [UIColor blackColor];
        [self addSubview:_photoImageView];
        
		// Setup
		self.backgroundColor = [UIColor blackColor];
		self.delegate = self;
		self.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    }
    
    return self;
}

- (XMTapDetectingImageView *)photoImageView {
    return _photoImageView;
}

- (void)layoutSubviews {
	_tapView.frame = self.bounds;
    
    [super layoutSubviews];
	
    if (_showPhotoHeightFull) {
        return;
    }
    
    // Center the image as it becomes smaller than the size of the screen
    CGSize boundsSize = self.bounds.size;
    CGRect frameToCenter = _photoImageView.frame;
    
    // Horizontally
    if (frameToCenter.size.width < boundsSize.width) {
        frameToCenter.origin.x = floorf((boundsSize.width - frameToCenter.size.width) / 2.0);
	} else {
        frameToCenter.origin.x = 0;
	}
    
    // Vertically
    if (frameToCenter.size.height < boundsSize.height) {
        frameToCenter.origin.y = floorf((boundsSize.height - frameToCenter.size.height) / 2.0);
	} else {
        frameToCenter.origin.y = 0;
	}
    
	// Center
	if (!CGRectEqualToRect(_photoImageView.frame, frameToCenter))
    {
		_photoImageView.frame = frameToCenter;
	}
}

#pragma mark - Setter
- (void)setPhoto:(id<XMPhotoProtocol>)photo {
    
    if (_photo != photo)
    {
        _photo = photo;
    }
    
    if (_photo)
    {
        [self displayImage];
    }
}

#pragma mark - PV
- (void)layoutPhotoLoadingView
{
    [_photoLoadingView removeFromSuperview];
    if ([_photo respondsToSelector:@selector(photoLoadingView)])
    {
        _photoLoadingView = (XMPhotoLoadingView *)[_photo photoLoadingView];
        _photoLoadingView.supportReload = self.isPhotoSupportedReload;
        [_photoLoadingView setFrame:_photoBrowser.view.bounds];
        _photoLoadingView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin |
        UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleRightMargin;
        [self setUserInteractionEnabled:NO];
        [self addSubview:_photoLoadingView];
    }
}

#pragma mark - PB
- (void)displayImageStartLoading
{
    if ([_photoLoadingView respondsToSelector:@selector(displayLoading)])
    {
        [_photoLoadingView displayLoading];
    }
}


- (UIImage *)imageForPhoto:(id<XMPhotoProtocol>)photo
{
    if (photo) {
		// Get image or obtain in background
		if ([photo underlyingImage]) {
			return [photo underlyingImage];
		} else {
            [photo loadUnderlyingImageAndNotify];
		}
	}
	return nil;
}

- (void)displayImage
{
    if (_photo)
    {
        [self setUserInteractionEnabled:YES];
		// Reset
		// Reset
		self.maximumZoomScale = 1;
		self.minimumZoomScale = 1;
		self.zoomScale = 1;
        self.contentSize = CGSizeMake(0, 0);
		
		// Get image from browser as it handles ordering of fetching
		UIImage *img = [self imageForPhoto:_photo];
		if (img)
        {
            [_photoLoadingView removeFromSuperview];
			
            // Set image
			_photoImageView.image = img;
            
            // Setup photo frame
			CGRect photoImageViewFrame;
			photoImageViewFrame.origin = CGPointZero;
			photoImageViewFrame.size = img.size;
			_photoImageView.frame = photoImageViewFrame;
			self.contentSize = photoImageViewFrame.size;
        }
        else
        {
             img = _photoImageView.image;
            [self layoutPhotoLoadingView];
		}
        
        if (img) {
            
            if (_showPhotoHeightFull) {
                [self resetSize:img.size];
            } else {
                // Set zoom to minimum zoom
                [self setMaxMinZoomScalesForCurrentBounds];
            }
        }
        

        [self setNeedsLayout];
	}
}

- (void)resetSize:(CGSize)imgSize {
    // Setup photo frame
    CGSize size =  self.bounds.size;
    CGFloat x = 0.0;
    CGFloat y = 0.0;
    if (imgSize.width <= size.width) {
        x = (size.width - imgSize.width)* 0.5;
    }
    if (imgSize.height <= size.height) {
        y = (size.height - imgSize.height) * 0.5;
    }
    CGRect photoImageViewFrame;
    photoImageViewFrame.origin = CGPointMake(x,y);
    photoImageViewFrame.size = imgSize;
    _photoImageView.frame = photoImageViewFrame;
    self.contentSize = imgSize;
}

- (void)setMaxMinZoomScalesForCurrentBounds
{
    // Reset
	self.maximumZoomScale = 1;
	self.minimumZoomScale = 1;
	self.zoomScale = 1;
    
	// Bail
	if (_photoImageView.image == nil) return;
    
	// Sizes
    CGSize boundsSize = self.bounds.size;
    CGSize imageSize = _photoImageView.frame.size;
    
    // Calculate Min
    CGFloat xScale = boundsSize.width / imageSize.width;    // the scale needed to perfectly fit the image width-wise
    CGFloat yScale = boundsSize.height / imageSize.height;  // the scale needed to perfectly fit the image height-wise
    CGFloat minScale = MIN(xScale, yScale);                 // use minimum of these to allow the image to become fully visible
    
	// If image is smaller than the screen then ensure we show it at
	// min scale of 1
	if (xScale > 1 && yScale > 1) {
		minScale = 1.0;
	}
    
	// Calculate Max
	CGFloat maxScale = 2.0; // Allow double scale
    // on high resolution screens we have double the pixel density, so we will be seeing every pixel if we limit the
    // maximum zoom scale to 0.5.
	if ([UIScreen instancesRespondToSelector:@selector(scale)]) {
		maxScale = maxScale / [[UIScreen mainScreen] scale];
	}
    
	// Set
	self.maximumZoomScale = maxScale;
	self.minimumZoomScale = minScale;
	self.zoomScale = minScale;
	zoomScaleFromInit = minScale;
	// Reset position
	_photoImageView.frame = CGRectMake(0, 0, _photoImageView.frame.size.width, _photoImageView.frame.size.height);
	[self setNeedsLayout];
}


- (void)displayImageFailure
{
    [self setUserInteractionEnabled:YES];
    [_photoLoadingView displayFailure];
}

- (void)prepareForReuse
{
    shouldSupportedPanGesture = NO;
    self.photo = nil;
}

#pragma mark - UIScrollViewDelegate
- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView {
	return _photoImageView;
}

- (void)scrollViewDidZoom:(UIScrollView *)scrollView
{
    shouldSupportedPanGesture = ((self.zoomScale == zoomScaleFromInit) && self.isPhotoSupportedPanGesture);
}

#pragma mark - Touch Event

- (void)view:(UIView *)view singleTapDetected:(UITouch *)touch {
    [_photoBrowser performSelector:@selector(toggleControls) withObject:nil afterDelay:0.2];
}

- (UIView *)hitTest:(CGPoint)point withEvent:(UIEvent *)event
{
    UIView *view = [super hitTest:point withEvent:event];
    
    return view;
}

- (void)handleImageViewSingleTap:(CGPoint)touchPoint {
    
	[_photoBrowser performSelector:@selector(toggleControls) withObject:nil afterDelay:0.2];
}

- (void)handleImageViewDoubleTap:(CGPoint)touchPoint {
    if (_showPhotoHeightFull) {
        return;
    }
    
    // Cancel any single tap handling
	[NSObject cancelPreviousPerformRequestsWithTarget:_photoBrowser];
    
	// Zoom
	if (self.zoomScale == self.maximumZoomScale) {
        
		// Zoom out
		[self setZoomScale:self.minimumZoomScale animated:YES];
        
	} else {
        
		// Zoom in
		[self zoomToRect:CGRectMake(touchPoint.x, touchPoint.y, 1, 1) animated:YES];
        
	}
    
	// Delay controls
}

#pragma mark - CXTapDetectingImageViewDelegate

- (void)imageView:(UIImageView *)imageView singleTapDetected:(UITouch *)touch {
    [self handleImageViewSingleTap:[touch locationInView:imageView]];
}
- (void)imageView:(UIImageView *)imageView doubleTapDetected:(UITouch *)touch {
    [self handleImageViewDoubleTap:[touch locationInView:imageView]];
}

- (void)imageView:(UIImageView *)imageView tripleTapDetected:(UITouch *)touch {
    [self handleImageViewDoubleTap:[touch locationInView:imageView]];
}

@end
